import os

from django.core.exceptions import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _


def validate_file_extension(value):
    valid_extensions = ['.pdf', '.doc', '.docx', '.jpg', '.png', '.xlsx', '.xls', '.mp3']
    file_extension = os.path.splitext(value.name)[-1]
    if not file_extension.lower() in valid_extensions:
        raise ValidationError(_("Unsupported file extension. valid extensions: {}".format(valid_extensions)))


def validate_file_size(value):
    limit = 10 * 1024 * 1024
    if value.size > limit:
        raise ValidationError(_("File too large. Size should not exceed {}.".format(filesizeformat(limit))))



import logging

from celery import shared_task
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.mail.backends.smtp import EmailBackend
from django.db import transaction
from django.template import Context, Template
from django.utils.translation import ugettext_lazy as _

from notification_system.models import OutgoingMessage, NotificationEvent
from notification_system.utils import get_user_common_data

logger = logging.getLogger('notification_system')


@shared_task()
def notification_system_send_internal_message(notification_event_id):
    send_email_status = True
    notification_event = NotificationEvent.objects.prefetch_related('notification_groups__members', 'attachments')
    notification_event = notification_event.prefetch_related('notification_groups')
    notification_event = notification_event.select_related('owner', 'template')
    notification_event = notification_event.get(id=notification_event_id)

    subject = notification_event.get_subject()
    content = notification_event.get_content()
    recipients = notification_event.get_recipients(check_email_isnull=True)

    outgoing_message_objects = []
    for user in recipients:
        context = get_user_common_data(user=user)
        recipient = user.email or ''
        outgoing_message_object = OutgoingMessage(subject=subject, recipient=recipient, context=dict(context),
                                                  content=content, user=user, notification_event=notification_event,
                                                  status=OutgoingMessage.STATUS_SENT,
                                                  outgoing_message_type=OutgoingMessage.TYPE_INTERNAL_MESSAGE)
        outgoing_message_objects.append(outgoing_message_object)

    if outgoing_message_objects:
        try:
            OutgoingMessage.objects.bulk_create(outgoing_message_objects)
        except Exception as e:
            message = _("Error occurred while sending internal messages. error_type: {}, error message: {}"
                        .format(type(e), str(e)))
            logger.error(msg=message)
            send_email_status = False

    return send_email_status


@shared_task(rate_limit=getattr(settings, 'NOTIFICATION_SYSTEM_SEND_EMAIL_RATE_LIMIT', '700/m'))
def notification_system_send_email(subject, recipient, email_from, context, content='', notification_event=None,
                                   user_id=None, smtp_config=None, attachments=None):
    send_email_status = True
    outgoing_message = OutgoingMessage.objects.create(recipient=recipient, subject=subject, context=dict(context),
                                                      content=content, user_id=user_id,
                                                      notification_event_id=notification_event)

    try:
        template_instance = Template(template_string=content)
        context_instance = Context(dict_=context)
        content = template_instance.render(context=context_instance)
        emails = recipient if isinstance(recipient, list) else [recipient]
        smtp_connection = EmailBackend(**smtp_config) if smtp_config else None
        headers = {'outgoing_message_id': outgoing_message.id}
        mail = EmailMultiAlternatives(subject=subject, body=content, from_email=email_from, to=emails, headers=headers,
                                      connection=smtp_connection)
        for attachment in attachments:
            mail.attach_file(path=attachment)

        mail.attach_alternative(content, "text/html")
        mail.send()

        outgoing_message.status = OutgoingMessage.STATUS_SENT

    except Exception as e:
        logger.error(_("Error occurred while sending email. error_type: {}, error message: {}".format(type(e), str(e))))
        send_email_status = False
        outgoing_message.status = OutgoingMessage.STATUS_FAILED
        outgoing_message.error_message = "type: {}\nmessage: {}".format(type(e), str(e))

    outgoing_message.save()
    return send_email_status


@shared_task()
def update_outgoing_message_status(outgoing_message_id, status):
    map_status = {
        'Bounce': OutgoingMessage.STATUS_AMAZON_BOUNCE,
        'Complaint': OutgoingMessage.STATUS_AMAZON_COMPLAINT,
        'Delivery': OutgoingMessage.STATUS_AMAZON_DELIVERY,
        'Send': OutgoingMessage.STATUS_AMAZON_SEND,
        'Reject': OutgoingMessage.STATUS_AMAZON_REJECT,
        'Open': OutgoingMessage.STATUS_AMAZON_OPEN,
        'Click': OutgoingMessage.STATUS_AMAZON_CLICK,
        'Rendering Failure': OutgoingMessage.STATUS_AMAZON_RENDERING_FAILURE,
        'DeliveryDelay': OutgoingMessage.STATUS_AMAZON_DELIVERY_DELAY,
        'Subscription': OutgoingMessage.STATUS_AMAZON_SUBSCRIPTION,
    }

    status = map_status.get(status)
    if (not status) or (not str(outgoing_message_id).isdigit()):
        return False

    with transaction.atomic():
        result = OutgoingMessage.objects.filter(pk=outgoing_message_id).update(status=status)

    return bool(result)



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class NotificationSystemConfig(AppConfig):
    default_auto_field = 'django.db.models.AutoField'
    name = 'notification_system'
    verbose_name = _("Notification System")



from django.conf import settings


def get_user_common_data(user):
    default_user_field = getattr(settings, 'NOTIFICATION_SYSTEM_DEFAULT_USER_FIELD',
                                 ['id', 'username', 'first_name', 'last_name', 'email'])
    return {key: getattr(user, key, '') for key in default_user_field}



# Generated by Django 3.2.12 on 2022-07-23 12:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('notification_system', '0003_auto_20220716_0639'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationevent',
            name='notification_event_type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Email'), (2, 'Internal message')], default=1, verbose_name='type'),
        ),
        migrations.AlterField(
            model_name='notificationevent',
            name='smtp_provider',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.smtpprovider', verbose_name='smtp provider'),
        ),
        migrations.AlterField(
            model_name='outgoingmessage',
            name='outgoing_message_type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Email'), (2, 'Internal message')], default=1, verbose_name='type'),
        ),
    ]



import abc

from django.conf import settings

from notification_system.tasks import notification_system_send_email


class NotificationHandler(metaclass=abc.ABCMeta):
    TYPE_EMAIL = 1
    TYPE_INTERNAL_MESSAGE = 2
    EMAIL_NOTIFICATION_QUEUE_NAME = getattr(settings, 'EMAIL_NOTIFICATION_QUEUE_NAME', 'send_email_notification')
    INTERNAL_MESSAGE_NOTIFICATION_QUEUE_NAME = getattr(settings, 'INTERNAL_MESSAGE_NOTIFICATION_QUEUE_NAME',
                                                       'send_internal_message_notification')

    @abc.abstractmethod
    def send(self):
        pass


class EmailNotificationHandler(NotificationHandler):

    def __init__(self, subject, recipient, email_from, context, notification_type=NotificationHandler.TYPE_EMAIL,
                 content='', notification_event=None, user_id=None, smtp_config=None, attachments=None):
        """
        :param subject: This field specifies the notification subject
        :type subject: str
        :param recipient: The recipient of the notification is specified in this field
        :type recipient: str
        :param email_from: In this field, you specify the email address of the sender
        :type email_from: str
        :param context: This field specifies extra info for notification
        :type context: dict
        :param content: This field specifies mail template.
        :type content: str
        :param notification_type: This field specifies the notification type
        :type notification_type: int
        :param notification_event: This field specifies the notification_event id
        :type notification_event: int
        :param user_id: This field specifies the user_id
        :type user_id: int
        :param smtp_config: This field specifies the smtp_config
        :type smtp_config: dict
        :param attachments: The attachments of this notification are specified in this field
        :type attachments: list
        """
        self.subject = subject
        self.recipient = recipient
        self.email_from = email_from
        self.context = context
        self.content = content
        self.notification_event = notification_event
        self.user_id = user_id
        self.smtp_config = smtp_config
        self.attachments = attachments
        self.notification_type = notification_type

    def send(self):
        kwargs = {
            'subject': self.subject,
            'recipient': self.recipient,
            'email_from': self.email_from,
            'context': self.context,
            'content': self.content,
            'notification_event': self.notification_event,
            'user_id': self.user_id,
            'smtp_config': self.smtp_config,
            'attachments': self.attachments,
        }
        notification_system_send_email.apply_async(kwargs=kwargs, queue=self.EMAIL_NOTIFICATION_QUEUE_NAME)


class InternalMessageNotificationHandler(NotificationHandler):

    def __init__(self, subject, recipients, context, notification_event, content, user_id,
                 notification_type=NotificationHandler.TYPE_INTERNAL_MESSAGE):
        """
        :param subject: This field specifies the notification subject
        :type subject: str
        :param recipients: The recipients of the notification is specified in this field
        :type recipients: list
        :param context: This field specifies extra info for notification
        :type context: dict
        :param content: This field specifies mail template.
        :type content: str
        :param notification_type: This field specifies the notification type
        :type notification_type: int
        :param notification_event: This field specifies the notification_event id
        :type notification_event: int
        :param user_id: This field specifies the user_id
        :type user_id: int
        """
        self.subject = subject
        self.recipients = recipients
        self.context = context
        self.content = content
        self.notification_event = notification_event
        self.user_id = user_id
        self.notification_type = notification_type

    def send(self):
        pass



# Generated by Django 3.2.12 on 2022-07-13 11:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import notification_system.validators


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('notification_system', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationevent',
            name='recipients',
            field=models.ManyToManyField(blank=True, related_name='recipients', to=settings.AUTH_USER_MODEL, verbose_name='members'),
        ),
        migrations.AlterField(
            model_name='notificationevent',
            name='notification_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.notificationgroup', verbose_name='notification group'),
        ),
        migrations.CreateModel(
            name='Media',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('file', models.FileField(upload_to='notification_system/media/file/', validators=[notification_system.validators.validate_file_extension, notification_system.validators.validate_file_size], verbose_name='file')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='media', to=settings.AUTH_USER_MODEL, verbose_name='owner')),
            ],
            options={
                'verbose_name': 'Media',
                'verbose_name_plural': 'Media',
            },
        ),
    ]



from django.conf import settings
from django.db.models import Q
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets, mixins, filters
from rest_framework.decorators import action
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework_simplejwt.authentication import JWTAuthentication

from notification_system.api.serializers import (NotificationEventSerializer, EmailTemplateSerializer,
                                                 OutgoingMessageSerializer, MinimalNotificationGroupSerializer,
                                                 MinimalSmtpProviderSerializer, OutgoingMessageNotificationSerializer,
                                                 MediaSerializer)
from notification_system.models import (NotificationEvent, EmailTemplate, OutgoingMessage, NotificationGroup,
                                        SmtpProvider, Media)
from notification_system.notification_delivery import broadcast_event_messages_to_recipients
from notification_system.paginations import CustomLimitOffsetPagination
from notification_system.parsers import PlainTextParser
from notification_system.permissions import CustomDjangoModelPermissions
from notification_system.tasks import update_outgoing_message_status


class NotificationEventViewSet(mixins.ListModelMixin,
                               mixins.RetrieveModelMixin,
                               mixins.CreateModelMixin,
                               mixins.UpdateModelMixin,
                               viewsets.GenericViewSet):
    """
    A ViewSet for viewing Notification Event.
    """
    queryset = NotificationEvent.actives.all()
    serializer_class = NotificationEventSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [DjangoFilterBackend, filters.OrderingFilter]
    ordering_fields = ['id']
    filter_fields = ['notification_groups', 'recipients', 'notification_event_type']

    def get_queryset(self):
        queryset = super().get_queryset()
        queryset = queryset.prefetch_related('attachments', 'notification_groups__members', 'notification_groups')
        queryset = queryset.prefetch_related('recipients').select_related('smtp_provider', 'template')
        return queryset.filter(owner=self.request.user)

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)

    @action(detail=True, methods=['POST'], url_path="broadcast-message")
    def broadcast_message(self, request, pk=None):
        instance = self.get_object()
        broadcast_event_messages_to_recipients(event_message_id=instance.id)
        return Response(status=200)


class EmailTemplateViewSet(mixins.ListModelMixin,
                           mixins.RetrieveModelMixin,
                           mixins.CreateModelMixin,
                           mixins.UpdateModelMixin,
                           viewsets.GenericViewSet):
    """
    A ViewSet for viewing Notification Event.
    """
    queryset = EmailTemplate.actives.select_related('owner').all()
    serializer_class = EmailTemplateSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [filters.OrderingFilter]
    ordering_fields = ['id']

    def get_queryset(self):
        return super().get_queryset().filter(owner=self.request.user)

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)


class OutgoingMessageViewSet(mixins.ListModelMixin,
                             mixins.RetrieveModelMixin,
                             viewsets.GenericViewSet):
    """
    A ViewSet for viewing outgoing message.
    """
    queryset = OutgoingMessage.objects.all()
    serializer_class = OutgoingMessageSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [DjangoFilterBackend, filters.OrderingFilter]
    ordering_fields = ['id']
    filter_fields = ['outgoing_message_type', 'status', 'notification_event']

    def get_queryset(self):
        queryset = super().get_queryset()
        queryset = queryset.prefetch_related('notification_event__attachments')
        queryset = queryset.select_related('user', 'notification_event', 'notification_event__template')
        return queryset.filter(Q(user=self.request.user) | Q(recipient=self.request.user.email)).distinct()


class NotificationGroupViewSet(mixins.CreateModelMixin,
                               mixins.UpdateModelMixin,
                               mixins.ListModelMixin,
                               mixins.RetrieveModelMixin,
                               viewsets.GenericViewSet):
    """
    A ViewSet for viewing notification group.
    """
    queryset = NotificationGroup.actives.select_related('owner').all()
    serializer_class = MinimalNotificationGroupSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [filters.OrderingFilter]
    ordering_fields = ['id']

    def get_queryset(self):
        return super().get_queryset().filter(owner=self.request.user)

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)


class SmtpProviderViewSet(mixins.ListModelMixin,
                          mixins.RetrieveModelMixin,
                          viewsets.GenericViewSet):
    """
    A ViewSet for viewing smtp provider.
    """
    queryset = SmtpProvider.actives.prefetch_related('users').all()
    serializer_class = MinimalSmtpProviderSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [filters.OrderingFilter]
    ordering_fields = ['id']

    def get_queryset(self):
        return super().get_queryset().filter(users__id__in=[self.request.user.id]).distinct()


class MedialViewSet(mixins.ListModelMixin,
                    mixins.RetrieveModelMixin,
                    mixins.CreateModelMixin,
                    viewsets.GenericViewSet):
    """
    A ViewSet for viewing user uploaded media.
    """
    queryset = Media.objects.select_related('owner').all()
    serializer_class = MediaSerializer
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated, CustomDjangoModelPermissions]
    pagination_class = CustomLimitOffsetPagination
    filter_backends = [filters.OrderingFilter]
    ordering_fields = ['id']

    def get_queryset(self):
        return super().get_queryset().filter(owner=self.request.user)

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)


class OutgoingMessageNotificationAPIView(APIView):
    parser_classes = [PlainTextParser, JSONParser]

    def post(self, request):
        serializer = OutgoingMessageNotificationSerializer(data=request.data, context={'request': request})
        serializer.is_valid(raise_exception=True)
        kwargs = {
            'outgoing_message_id': serializer.data.get('outgoing_message_id'),
            'status': serializer.data.get('notificationType')
        }
        update_outgoing_message_status_queue_name = getattr(settings, 'UPDATE_OUTGOING_MESSAGE_STATUS_QUEUE_NAME',
                                                            'update_outgoing_message_status')
        update_outgoing_message_status.apply_async(kwargs=kwargs, queue=update_outgoing_message_status_queue_name)
        return Response(serializer.data)



# Generated by Django 3.2.12 on 2022-07-16 06:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification_system', '0002_auto_20220713_1151'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='notificationevent',
            name='notification_group',
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='notification_groups',
            field=models.ManyToManyField(blank=True, related_name='notification_events', to='notification_system.NotificationGroup', verbose_name='notification groups'),
        ),
    ]



import logging

from django.contrib.auth import get_user_model
from django.template import Context, Template
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers, exceptions

from notification_system.models import (NotificationEvent, EmailTemplate, Attachment, SmtpProvider, NotificationGroup,
                                        OutgoingMessage, Media)
from notification_system.utils import get_user_common_data

User = get_user_model()

logger = logging.getLogger('notification_system')


class AttachmentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Attachment
        fields = ['id', 'file']


class EmailTemplateSerializer(serializers.ModelSerializer):
    class Meta:
        model = EmailTemplate
        fields = ['id', 'title', 'subject', 'content']


class MediaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Media
        fields = ['id', 'file']


class MinimalSmtpProviderSerializer(serializers.ModelSerializer):
    class Meta:
        model = SmtpProvider
        fields = ['id', 'provider_name']


class MinimalNotificationGroupSerializer(serializers.ModelSerializer):
    members = serializers.PrimaryKeyRelatedField(many=True, queryset=User.objects.filter(is_active=True),
                                                 required=False)

    class Meta:
        model = NotificationGroup
        fields = ['id', 'name', 'members']


class NotificationEventSerializer(serializers.ModelSerializer):
    attachments = AttachmentSerializer(many=True, required=False)
    template_detail = EmailTemplateSerializer(source='template', read_only=True)
    smtp_provider_detail = MinimalSmtpProviderSerializer(source='smtp_provider', read_only=True)
    notification_groups = serializers.PrimaryKeyRelatedField(many=True, queryset=NotificationGroup.actives.all(),
                                                             required=False, write_only=True)
    notification_event_type_display = serializers.CharField(source='get_notification_event_type_display',
                                                            read_only=True)
    notification_groups_detail = MinimalNotificationGroupSerializer(source='notification_groups', many=True,
                                                                    read_only=True)
    recipients = serializers.PrimaryKeyRelatedField(many=True, queryset=User.objects.filter(is_active=True),
                                                    required=False)

    class Meta:
        model = NotificationEvent
        fields = ['id', 'title', 'smtp_provider', 'smtp_provider_detail', 'template', 'template_detail', 'subject',
                  'content', 'notification_event_type', 'notification_event_type_display', 'notification_groups',
                  'notification_groups_detail', 'attachments', 'recipients']

        extra_kwargs = {
            'template': {'write_only': True, 'allow_null': True},
            'smtp_provider': {'write_only': True},
        }

    def create(self, validated_data):
        attachments = validated_data.pop('attachments', list())
        instance = super().create(validated_data)

        attachment_list = []
        for attachment in attachments:
            attachment_list.append(Attachment(file=attachment.get('file'), notification_event=instance))

        if attachment_list:
            instance.attachments.bulk_create(attachment_list)

        return instance

    def update(self, instance, validated_data):
        attachments = validated_data.pop('attachments', list())
        instance = super().update(instance, validated_data)

        if attachments:
            instance.attachments.all().delete()

        attachment_list = []
        for attachment in attachments:
            attachment_list.append(Attachment(file=attachment.get('file'), notification_event=instance))

        if attachment_list:
            instance.attachments.bulk_create(attachment_list)

        return instance

    def validate_template(self, value):
        if value and ((not value.is_active) or (not self.context['request'].user == value.owner)):
            raise exceptions.ValidationError(detail=_("You do not have permission to access this template."))
        return value

    def validate_notification_groups(self, value):
        user = self.context['request'].user
        for notification_group in value:
            if not notification_group.is_active or notification_group.owner != user:
                message = _("You do not have permission to access notification group with id: {}.")
                raise exceptions.ValidationError(detail=message.format(notification_group.id))

        return value

    def validate_smtp_provider(self, value):
        if value:
            if (not value.is_active) or (not value.users.filter(pk__in=[self.context['request'].user.id]).exists()):
                raise exceptions.ValidationError(detail=_("You do not have permission to access this smtp provider."))
        return value

    def validate(self, attrs):
        attrs = super().validate(attrs=attrs)
        notification_event_type = attrs.get("notification_event_type")
        smtp_provider = attrs.get("smtp_provider")

        if notification_event_type == NotificationEvent.TYPE_EMAIL and not smtp_provider:
            raise exceptions.ValidationError({"smtp_provider": _("This field is required.")})

        return attrs


class MinimalNotificationEventSerializer(serializers.ModelSerializer):
    template_detail = EmailTemplateSerializer(source='template', read_only=True)
    attachments = AttachmentSerializer(many=True, read_only=True)
    notification_event_type_display = serializers.CharField(source='get_notification_event_type_display',
                                                            read_only=True)

    class Meta:
        model = NotificationEvent
        fields = ['id', 'title', 'subject', 'content', 'notification_event_type', 'notification_event_type_display',
                  'template_detail', 'attachments']


class OutgoingMessageSerializer(serializers.ModelSerializer):
    status_display = serializers.CharField(source="get_status_display", read_only=True)
    outgoing_message_type_display = serializers.CharField(source="get_outgoing_message_type_display", read_only=True)
    notification_event_detail = MinimalNotificationEventSerializer(source='notification_event', read_only=True)
    rendered_content = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = OutgoingMessage
        fields = ['id', 'notification_event_detail', 'recipient', 'subject', 'status', 'status_display', 'content',
                  'rendered_content', 'outgoing_message_type', 'outgoing_message_type_display', 'created_time',
                  'updated_time']

    def get_rendered_content(self, obj):
        template_instance = Template(template_string=obj.content or '')
        context = get_user_common_data(user=obj.user)
        context_instance = Context(dict_=context)
        content = template_instance.render(context=context_instance)
        return content


class OutgoingMessageNotificationSerializer(serializers.Serializer):
    Type = serializers.CharField(required=False)
    notificationType = serializers.CharField(required=False)
    mail = serializers.DictField(write_only=True, required=False)
    outgoing_message_id = serializers.IntegerField(read_only=True)

    def validate(self, attrs):
        attrs = super().validate(attrs=attrs)
        flag_check_condition = True

        if attrs.get('Type') == "SubscriptionConfirmation":
            msg = _("Amazon SubscriptionConfirmation webhook received. data: {}".format(self.context['request'].data))
            logger.info(msg=msg)
            raise exceptions.ParseError(_("The payload is not valid."))

        for mail_header in attrs.get('mail', dict()).get('headers', []):
            name = mail_header.get('name')
            value = mail_header.get('value')
            if (name == 'outgoing_message_id') and value:
                if value.isdigit():
                    attrs['outgoing_message_id'] = value
                    flag_check_condition = False

        if flag_check_condition:
            raise exceptions.ParseError(_("The payload is not valid."))

        return attrs



# Generated by Django 3.2.12 on 2022-05-14 10:26

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import notification_system.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('title', models.CharField(max_length=255, verbose_name='template title')),
                ('subject', models.CharField(max_length=255, verbose_name='subject')),
                ('content', models.TextField(blank=True, verbose_name='content')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active?')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='email_templates', to=settings.AUTH_USER_MODEL, verbose_name='owner')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NotificationEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('title', models.CharField(max_length=255, verbose_name='notification events title')),
                ('subject', models.CharField(blank=True, max_length=255, verbose_name='subject')),
                ('content', models.TextField(blank=True, verbose_name='content')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active?')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SmtpProvider',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('provider_name', models.CharField(max_length=255, verbose_name='provider name')),
                ('email_host', models.CharField(max_length=255, verbose_name='email host')),
                ('email_host_user', models.CharField(max_length=255, verbose_name='email host user')),
                ('email_host_password', models.CharField(max_length=1024, verbose_name='email host password')),
                ('email_port', models.CharField(default=587, max_length=255, verbose_name='email port')),
                ('email_from', models.CharField(blank=True, help_text="Mail from <'mail@mail.com'>", max_length=255, verbose_name='email from')),
                ('email_timeout', models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='email timeout')),
                ('email_use_tls', models.BooleanField(default=False, verbose_name='email use tls')),
                ('email_use_ssl', models.BooleanField(default=False, verbose_name='email use ssl')),
                ('email_ssl_keyfile', models.TextField(blank=True, verbose_name='email ssl keyfile')),
                ('email_ssl_certfile', models.TextField(blank=True, verbose_name='email ssl certfile')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active?')),
                ('users', models.ManyToManyField(blank=True, related_name='users', to=settings.AUTH_USER_MODEL, verbose_name='user access')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OutgoingMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('recipient', models.CharField(blank=True, db_index=True, max_length=255, verbose_name='recipient')),
                ('subject', models.CharField(blank=True, max_length=255, verbose_name='subject')),
                ('content', models.TextField(blank=True, verbose_name='message content')),
                ('context', models.TextField(blank=True, verbose_name='message context')),
                ('error_message', models.TextField(blank=True, verbose_name='error message')),
                ('status', models.PositiveSmallIntegerField(choices=[(3, 'Failed'), (2, 'Pending'), (1, 'Sent'), (4, 'Bounce'), (5, 'Complaint'), (6, 'Delivery'), (7, 'Send'), (8, 'Reject'), (9, 'Open'), (10, 'Click'), (11, 'Rendering Failure'), (12, 'Delivery Delay'), (13, 'Subscription')], default=2, verbose_name='status')),
                ('outgoing_message_type', models.PositiveSmallIntegerField(choices=[(1, 'Email')], default=1, verbose_name='type')),
                ('notification_event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='outgoing_messages', to='notification_system.notificationevent', verbose_name='notification event')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='outgoing_messages', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NotificationGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('name', models.CharField(max_length=255, verbose_name='name')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active?')),
                ('members', models.ManyToManyField(blank=True, related_name='members', to=settings.AUTH_USER_MODEL, verbose_name='members')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='notification_groups', to=settings.AUTH_USER_MODEL, verbose_name='owner')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='notification_group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.notificationgroup', verbose_name='notification group'),
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to=settings.AUTH_USER_MODEL, verbose_name='owner'),
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='smtp_provider',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.smtpprovider', verbose_name='smtp provider'),
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='template',
            field=models.ForeignKey(blank=True, help_text='If template is selected, HTML message and subject fields will not be used- they will be populated from template', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.emailtemplate'),
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('file', models.FileField(upload_to='notification_system/attachment/file/', validators=[notification_system.validators.validate_file_extension, notification_system.validators.validate_file_size], verbose_name='file')),
                ('notification_event', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='attachments', to='notification_system.notificationevent', verbose_name='notification event')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.urls import path
from rest_framework import routers

from notification_system.api.views import (NotificationEventViewSet, EmailTemplateViewSet, OutgoingMessageViewSet,
                                           OutgoingMessageNotificationAPIView, NotificationGroupViewSet,
                                           SmtpProviderViewSet, MedialViewSet)

app_name = "notification_events"

router = routers.SimpleRouter()
router.register("notification-events", NotificationEventViewSet, basename="notification_events")
router.register("email-templates", EmailTemplateViewSet, basename="email_templates")
router.register("outgoing-messages", OutgoingMessageViewSet, basename="outgoing_messages")
router.register("notification-groups", NotificationGroupViewSet, basename="notification_groups")
router.register("smtp-providers", SmtpProviderViewSet, basename="smtp_providers")
router.register("media", MedialViewSet, basename="media")

urlpatterns = router.urls

urlpatterns += [
    path("outgoing-message-webhook/", OutgoingMessageNotificationAPIView.as_view(), name="outgoing_message_webhook"),
]



from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from notification_system.managers import ActiveManager
from notification_system.validators import validate_file_extension, validate_file_size

User = get_user_model()


class BaseModel(models.Model):
    created_time = models.DateTimeField(verbose_name=_('created time'), auto_now_add=True, db_index=True)
    updated_time = models.DateTimeField(verbose_name=_('updated time'), auto_now=True)

    class Meta:
        abstract = True


class SmtpProvider(BaseModel):
    provider_name = models.CharField(verbose_name=_('provider name'), max_length=255)
    email_host = models.CharField(verbose_name=_('email host'), max_length=255)
    email_host_user = models.CharField(verbose_name=_('email host user'), max_length=255)
    email_host_password = models.CharField(verbose_name=_('email host password'), max_length=1024)
    email_port = models.CharField(verbose_name=_('email port'), max_length=255, default=587)
    email_from = models.CharField(max_length=255, verbose_name=_('email from'), blank=True,
                                  help_text=mark_safe("Mail from <'mail@mail.com'>"))
    email_timeout = models.PositiveSmallIntegerField(verbose_name=_('email timeout'), blank=True, null=True)
    email_use_tls = models.BooleanField(verbose_name=_('email use tls'), default=False)
    email_use_ssl = models.BooleanField(verbose_name=_('email use ssl'), default=False)
    email_ssl_keyfile = models.TextField(verbose_name='email ssl keyfile', blank=True)
    email_ssl_certfile = models.TextField(verbose_name='email ssl certfile', blank=True)
    users = models.ManyToManyField(to=User, verbose_name=_('user access'), related_name='users', blank=True)
    is_active = models.BooleanField(verbose_name=_('is active?'), default=True)

    objects = models.Manager()
    actives = ActiveManager()

    def __str__(self):
        return "{} - {}".format(self.id, self.provider_name)

    def clean(self):
        if self.email_use_ssl and self.email_use_tls:
            raise ValidationError(
                _("email_use_tls/email_use_ssl are mutually exclusive, so only set one of those settings to True."))

    def get_smtp_config(self):
        return {
            "host": self.email_host,
            "username": self.email_host_user,
            "password": self.email_host_password,
            "port": self.email_port,
            "use_tls": self.email_use_tls,
            "use_ssl": self.email_use_ssl,
            "timeout": self.email_timeout,
            "ssl_keyfile": self.email_ssl_keyfile,
            "ssl_certfile": self.email_ssl_certfile,
        }


class NotificationGroup(BaseModel):
    name = models.CharField(verbose_name=_('name'), max_length=255)
    owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='notification_groups',
                              on_delete=models.PROTECT)
    members = models.ManyToManyField(to=User, verbose_name=_('members'), related_name='members', blank=True)
    is_active = models.BooleanField(verbose_name=_('is active?'), default=True)

    objects = models.Manager()
    actives = ActiveManager()

    def __str__(self):
        return "{} - {}".format(self.id, self.name)


class EmailTemplate(BaseModel):
    title = models.CharField(verbose_name=_('template title'), max_length=255)
    subject = models.CharField(verbose_name=_('subject'), max_length=255)
    content = models.TextField(verbose_name=_('content'), blank=True)
    owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='email_templates',
                              on_delete=models.PROTECT)
    is_active = models.BooleanField(verbose_name=_('is active?'), default=True)

    objects = models.Manager()
    actives = ActiveManager()

    def __str__(self):
        return "{} - {}".format(self.id, self.title)


class NotificationEvent(BaseModel):
    TYPE_EMAIL = 1
    TYPE_INTERNAL_MESSAGE = 2

    TYPE_CHOICES = (
        (TYPE_EMAIL, _('Email')),
        (TYPE_INTERNAL_MESSAGE, _('Internal message')),
    )

    title = models.CharField(verbose_name=_('notification events title'), max_length=255)
    smtp_provider = models.ForeignKey(to=SmtpProvider, verbose_name=_('smtp provider'), on_delete=models.PROTECT,
                                      related_name='notification_events', null=True, blank=True)
    owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='notification_events',
                              on_delete=models.PROTECT)
    notification_groups = models.ManyToManyField(to=NotificationGroup, verbose_name=_('notification groups'),
                                                 blank=True, related_name='notification_events')
    template = models.ForeignKey(to=EmailTemplate, on_delete=models.PROTECT, related_name='notification_events',
                                 null=True, blank=True,
                                 help_text=_("If template is selected, HTML message and subject fields will not be used"
                                             "- they will be populated from template"))
    notification_event_type = models.PositiveSmallIntegerField(verbose_name=_('type'), choices=TYPE_CHOICES,
                                                               default=TYPE_EMAIL)
    recipients = models.ManyToManyField(to=User, verbose_name=_('members'), related_name='recipients', blank=True)
    subject = models.CharField(verbose_name=_('subject'), max_length=255, blank=True)
    content = models.TextField(verbose_name=_('content'), blank=True)
    is_active = models.BooleanField(verbose_name=_('is active?'), default=True)

    objects = models.Manager()
    actives = ActiveManager()

    def __str__(self):
        return "{} - {}".format(self.id, self.title)

    def save(self, *args, **kwargs):
        self.smtp_provider = self.smtp_provider if self.notification_event_type == self.TYPE_EMAIL else None

        super().save(*args, **kwargs)

    def clean(self):
        if self.notification_event_type == self.TYPE_EMAIL and not self.smtp_provider:
            raise ValidationError(_("For this type, the SMTP provider field must be filled in."))

    def get_content(self):
        return self.content if not self.template else self.template.content

    def get_subject(self):
        return self.subject if not self.template else self.template.subject

    def get_recipients(self, check_email_isnull=False):
        recipients = self.recipients.all()

        for notification_group in self.notification_groups.filter(is_active=True):
            recipients = recipients | notification_group.members.all()

        filter_params = {
            "email__isnull": False,
        }
        if check_email_isnull:
            filter_params.pop('email__isnull', None)

        return recipients.filter(is_active=True, **filter_params).distinct()


class Attachment(BaseModel):
    file = models.FileField(verbose_name=_('file'), upload_to='notification_system/attachment/file/',
                            validators=[validate_file_extension, validate_file_size])
    notification_event = models.ForeignKey(to=NotificationEvent, verbose_name=_('notification event'),
                                           related_name='attachments', on_delete=models.PROTECT)

    def __str__(self):
        return str(self.id)


class Media(BaseModel):
    file = models.FileField(verbose_name=_('file'), upload_to='notification_system/media/file/',
                            validators=[validate_file_extension, validate_file_size])
    owner = models.ForeignKey(to=User, verbose_name=_('owner'), related_name='media', on_delete=models.PROTECT)

    class Meta:
        verbose_name = _('Media')
        verbose_name_plural = _('Media')

    def __str__(self):
        return str(self.id)


class OutgoingMessage(BaseModel):
    TYPE_EMAIL = 1
    TYPE_INTERNAL_MESSAGE = 2

    TYPE_CHOICES = (
        (TYPE_EMAIL, _('Email')),
        (TYPE_INTERNAL_MESSAGE, _('Internal message')),
    )

    STATUS_SENT = 1
    STATUS_PENDING = 2
    STATUS_FAILED = 3
    STATUS_AMAZON_BOUNCE = 4
    STATUS_AMAZON_COMPLAINT = 5
    STATUS_AMAZON_DELIVERY = 6
    STATUS_AMAZON_SEND = 7
    STATUS_AMAZON_REJECT = 8
    STATUS_AMAZON_OPEN = 9
    STATUS_AMAZON_CLICK = 10
    STATUS_AMAZON_RENDERING_FAILURE = 11
    STATUS_AMAZON_DELIVERY_DELAY = 12
    STATUS_AMAZON_SUBSCRIPTION = 13

    STATUS_CHOICES = [
        (STATUS_FAILED, _('Failed')),
        (STATUS_PENDING, _('Pending')),
        (STATUS_SENT, _('Sent')),
        (STATUS_AMAZON_BOUNCE, _('Bounce')),
        (STATUS_AMAZON_COMPLAINT, _('Complaint')),
        (STATUS_AMAZON_DELIVERY, _('Delivery')),
        (STATUS_AMAZON_SEND, _('Send')),
        (STATUS_AMAZON_REJECT, _('Reject')),
        (STATUS_AMAZON_OPEN, _('Open')),
        (STATUS_AMAZON_CLICK, _('Click')),
        (STATUS_AMAZON_RENDERING_FAILURE, _('Rendering Failure')),
        (STATUS_AMAZON_DELIVERY_DELAY, _('Delivery Delay')),
        (STATUS_AMAZON_SUBSCRIPTION, _('Subscription')),
    ]

    user = models.ForeignKey(to=User, verbose_name=_('user'), related_name='outgoing_messages', null=True, blank=True,
                             on_delete=models.PROTECT)
    notification_event = models.ForeignKey(to=NotificationEvent, verbose_name=_('notification event'), null=True,
                                           blank=True, related_name='outgoing_messages', on_delete=models.PROTECT)
    recipient = models.CharField(verbose_name=_('recipient'), max_length=255, blank=True, db_index=True)
    subject = models.CharField(verbose_name=_('subject'), max_length=255, blank=True)
    content = models.TextField(verbose_name=_('message content'), blank=True)
    context = models.TextField(verbose_name=_('message context'), blank=True)
    error_message = models.TextField(verbose_name=_('error message'), blank=True)
    status = models.PositiveSmallIntegerField(verbose_name=_('status'), choices=STATUS_CHOICES, default=STATUS_PENDING)
    outgoing_message_type = models.PositiveSmallIntegerField(verbose_name=_('type'), choices=TYPE_CHOICES,
                                                             default=TYPE_EMAIL)

    def __str__(self):
        return "{} - {}".format(self.id, self.subject)



from django.contrib import admin

from notification_system.models import (SmtpProvider, EmailTemplate, Attachment, NotificationGroup, NotificationEvent,
                                        OutgoingMessage, Media)


@admin.register(NotificationGroup)
class NotificationGroupAdmin(admin.ModelAdmin):
    list_display = ['id', 'name', 'owner', 'is_active', 'created_time', 'updated_time']
    list_filter = ['is_active', 'created_time', 'updated_time']
    search_fields = ['name']
    filter_horizontal = ['members']
    raw_id_fields = ['owner']
    readonly_fields = ['created_time', 'updated_time']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.prefetch_related('members').select_related('owner').all()


@admin.register(SmtpProvider)
class SmtpProviderAdmin(admin.ModelAdmin):
    list_display = ['id', 'provider_name', 'email_host', 'email_host_user', 'email_port', 'email_use_tls',
                    'email_use_ssl', 'is_active']
    list_filter = ['is_active', 'email_use_tls', 'email_use_ssl', 'created_time', 'updated_time']
    search_fields = ['provider_name']
    filter_horizontal = ['users']
    readonly_fields = ['created_time', 'updated_time']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.prefetch_related('users').all()


@admin.register(EmailTemplate)
class EmailTemplateAdmin(admin.ModelAdmin):
    list_display = ['id', 'title', 'owner', 'subject', 'is_active', 'created_time', 'updated_time']
    list_filter = ['is_active', 'created_time', 'updated_time']
    readonly_fields = ['created_time', 'updated_time']
    raw_id_fields = ['owner']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related('owner').all()


@admin.register(Attachment)
class AttachmentAdmin(admin.ModelAdmin):
    list_display = ['id', 'file', 'notification_event', 'created_time', 'updated_time']
    list_filter = ['created_time', 'updated_time']
    raw_id_fields = ['notification_event']
    readonly_fields = ['created_time', 'updated_time']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related('notification_event').all()


class AttachmentInline(admin.TabularInline):
    model = Attachment


@admin.register(NotificationEvent)
class NotificationEventAdmin(admin.ModelAdmin):
    list_display = ['id', 'title', 'owner', 'smtp_provider', 'template', 'is_active', 'created_time', 'updated_time']
    list_filter = ['is_active', 'created_time', 'updated_time']
    filter_horizontal = ['recipients', 'notification_groups']
    raw_id_fields = ['owner', 'smtp_provider', 'template']
    readonly_fields = ['created_time', 'updated_time']
    inlines = [AttachmentInline]

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = queryset.prefetch_related('attachments', 'recipients', 'notification_groups')
        queryset = queryset.select_related('template', 'smtp_provider', 'owner').all()
        return queryset


@admin.register(OutgoingMessage)
class OutgoingMessageAdmin(admin.ModelAdmin):
    list_display = ['id', 'notification_event', 'user', 'recipient', 'subject', 'status', 'outgoing_message_type',
                    'created_time', 'updated_time']
    list_filter = ['status', 'outgoing_message_type', 'created_time', 'updated_time']
    raw_id_fields = ['user', 'notification_event']
    readonly_fields = ['created_time', 'updated_time']
    search_fields = ['recipient']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related('user', 'notification_event').all()


@admin.register(Media)
class MediaAdmin(admin.ModelAdmin):
    list_display = ['id', 'file', 'owner', 'created_time', 'updated_time']
    list_filter = ['created_time', 'updated_time']
    raw_id_fields = ['owner']
    readonly_fields = ['created_time', 'updated_time']

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related('owner').all()



from notification_system.common.notification_handler import EmailNotificationHandler, NotificationHandler
from notification_system.models import NotificationEvent
from notification_system.tasks import notification_system_send_internal_message
from notification_system.utils import get_user_common_data


def broadcast_event_messages_to_recipients(event_message_id):
    notification_event = NotificationEvent.objects.prefetch_related('notification_groups__members', 'attachments')
    notification_event = notification_event.prefetch_related('notification_groups')
    notification_event = notification_event.select_related('smtp_provider', 'owner', 'template')
    notification_event = notification_event.get(id=event_message_id)

    if notification_event.notification_event_type == NotificationEvent.TYPE_EMAIL:
        subject = notification_event.get_subject()
        html_content = notification_event.get_content()
        recipients = notification_event.get_recipients()
        email_from = notification_event.smtp_provider.email_from
        smtp_config = notification_event.smtp_provider.get_smtp_config()
        attachments = [attachment.file.path for attachment in notification_event.attachments.all()]

        for user in recipients:
            context = get_user_common_data(user=user)
            notification_handler = EmailNotificationHandler(subject=subject, recipient=user.email,
                                                            email_from=email_from,
                                                            context=context, content=html_content, user_id=user.id,
                                                            smtp_config=smtp_config, attachments=attachments,
                                                            notification_event=notification_event.id)
            notification_handler.send()

    else:
        queue_name = NotificationHandler.INTERNAL_MESSAGE_NOTIFICATION_QUEUE_NAME
        notification_system_send_internal_message.apply_async(args=(notification_event.pk,), queue=queue_name)

    return True
